@@ -1,22 +1,28 @@
 import asyncio
 import functools
+import warnings
+from collections.abc import Set
 from copy import deepcopy
+from json import JSONDecodeError
 
 import sentry_sdk
-from sentry_sdk._types import TYPE_CHECKING
 from sentry_sdk.consts import OP
-from sentry_sdk.integrations import DidNotEnable, Integration
+from sentry_sdk.integrations import (
+    DidNotEnable,
+    Integration,
+    _DEFAULT_FAILED_REQUEST_STATUS_CODES,
+)
 from sentry_sdk.integrations._wsgi_common import (
-    _in_http_status_code_range,
+    DEFAULT_HTTP_METHODS_TO_CAPTURE,
+    HttpCodeRangeContainer,
     _is_json_content_type,
     request_body_within_bounds,
 )
 from sentry_sdk.integrations.asgi import SentryAsgiMiddleware
-from sentry_sdk.scope import Scope, should_send_default_pii
+from sentry_sdk.scope import should_send_default_pii
 from sentry_sdk.tracing import (
     SOURCE_FOR_STYLE,
-    TRANSACTION_SOURCE_COMPONENT,
-    TRANSACTION_SOURCE_ROUTE,
+    TransactionSource,
 )
 from sentry_sdk.utils import (
     AnnotatedValue,
@@ -28,8 +34,10 @@ from sentry_sdk.utils import (
     transaction_from_function,
 )
 
+from typing import TYPE_CHECKING
+
 if TYPE_CHECKING:
-    from typing import Any, Awaitable, Callable, Dict, Optional, Tuple
+    from typing import Any, Awaitable, Callable, Container, Dict, Optional, Tuple, Union
 
     from sentry_sdk._types import Event, HttpStatusCodeRange
 
@@ -57,7 +65,12 @@ except ImportError:
 
 try:
     # Optional dependency of Starlette to parse form data.
-    import multipart  # type: ignore
+    try:
+        # python-multipart 0.0.13 and later
+        import python_multipart as multipart  # type: ignore
+    except ImportError:
+        # python-multipart 0.0.12 and earlier
+        import multipart  # type: ignore
 except ImportError:
     multipart = None
 
@@ -73,17 +86,41 @@ class StarletteIntegration(Integration):
 
     transaction_style = ""
 
-    def __init__(self, transaction_style="url", failed_request_status_codes=None):
-        # type: (str, Optional[list[HttpStatusCodeRange]]) -> None
+    def __init__(
+        self,
+        transaction_style="url",  # type: str
+        failed_request_status_codes=_DEFAULT_FAILED_REQUEST_STATUS_CODES,  # type: Union[Set[int], list[HttpStatusCodeRange], None]
+        middleware_spans=True,  # type: bool
+        http_methods_to_capture=DEFAULT_HTTP_METHODS_TO_CAPTURE,  # type: tuple[str, ...]
+    ):
+        # type: (...) -> None
         if transaction_style not in TRANSACTION_STYLE_VALUES:
             raise ValueError(
                 "Invalid value for transaction_style: %s (must be in %s)"
                 % (transaction_style, TRANSACTION_STYLE_VALUES)
             )
         self.transaction_style = transaction_style
-        self.failed_request_status_codes = failed_request_status_codes or [
-            range(500, 599)
-        ]
+        self.middleware_spans = middleware_spans
+        self.http_methods_to_capture = tuple(map(str.upper, http_methods_to_capture))
+
+        if isinstance(failed_request_status_codes, Set):
+            self.failed_request_status_codes = (
+                failed_request_status_codes
+            )  # type: Container[int]
+        else:
+            warnings.warn(
+                "Passing a list or None for failed_request_status_codes is deprecated. "
+                "Please pass a set of int instead.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
+            if failed_request_status_codes is None:
+                self.failed_request_status_codes = _DEFAULT_FAILED_REQUEST_STATUS_CODES
+            else:
+                self.failed_request_status_codes = HttpCodeRangeContainer(
+                    failed_request_status_codes
+                )
 
     @staticmethod
     def setup_once():
@@ -110,7 +147,7 @@ def _enable_span_for_middleware(middleware_class):
     async def _create_span_call(app, scope, receive, send, **kwargs):
         # type: (Any, Dict[str, Any], Callable[[], Awaitable[Dict[str, Any]]], Callable[[Dict[str, Any]], Awaitable[None]], Any) -> None
         integration = sentry_sdk.get_client().get_integration(StarletteIntegration)
-        if integration is None:
+        if integration is None or not integration.middleware_spans:
             return await old_call(app, scope, receive, send, **kwargs)
 
         middleware_name = app.__class__.__name__
@@ -118,14 +155,14 @@ def _enable_span_for_middleware(middleware_class):
         # Update transaction name with middleware name
         name, source = _get_transaction_from_middleware(app, scope, integration)
         if name is not None:
-            Scope.get_current_scope().set_transaction_name(
+            sentry_sdk.get_current_scope().set_transaction_name(
                 name,
                 source=source,
             )
 
         with sentry_sdk.start_span(
             op=OP.MIDDLEWARE_STARLETTE,
-            description=middleware_name,
+            name=middleware_name,
             origin=StarletteIntegration.origin,
         ) as middleware_span:
             middleware_span.set_tag("starlette.middleware_name", middleware_name)
@@ -135,7 +172,7 @@ def _enable_span_for_middleware(middleware_class):
                 # type: (*Any, **Any) -> Any
                 with sentry_sdk.start_span(
                     op=OP.MIDDLEWARE_STARLETTE_RECEIVE,
-                    description=getattr(receive, "__qualname__", str(receive)),
+                    name=getattr(receive, "__qualname__", str(receive)),
                     origin=StarletteIntegration.origin,
                 ) as span:
                     span.set_tag("starlette.middleware_name", middleware_name)
@@ -150,7 +187,7 @@ def _enable_span_for_middleware(middleware_class):
                 # type: (*Any, **Any) -> Any
                 with sentry_sdk.start_span(
                     op=OP.MIDDLEWARE_STARLETTE_SEND,
-                    description=getattr(send, "__qualname__", str(send)),
+                    name=getattr(send, "__qualname__", str(send)),
                     origin=StarletteIntegration.origin,
                 ) as span:
                     span.set_tag("starlette.middleware_name", middleware_name)
@@ -213,15 +250,14 @@ def patch_exception_middleware(middleware_class):
 
                 exp = args[0]
 
-                is_http_server_error = (
-                    hasattr(exp, "status_code")
-                    and isinstance(exp.status_code, int)
-                    and _in_http_status_code_range(
-                        exp.status_code, integration.failed_request_status_codes
+                if integration is not None:
+                    is_http_server_error = (
+                        hasattr(exp, "status_code")
+                        and isinstance(exp.status_code, int)
+                        and exp.status_code in integration.failed_request_status_codes
                     )
-                )
-                if is_http_server_error:
-                    _capture_exception(exp, handled=True)
+                    if is_http_server_error:
+                        _capture_exception(exp, handled=True)
 
                 # Find a matching handler
                 old_handler = None
@@ -292,7 +328,7 @@ def _add_user_to_sentry_scope(scope):
     if email:
         user_info.setdefault("email", starlette_user.email)
 
-    sentry_scope = Scope.get_isolation_scope()
+    sentry_scope = sentry_sdk.get_isolation_scope()
     sentry_scope.user = user_info
 
 
@@ -327,13 +363,13 @@ def patch_middlewares():
 
     if not_yet_patched:
 
-        def _sentry_middleware_init(self, cls, **options):
-            # type: (Any, Any, Any) -> None
+        def _sentry_middleware_init(self, cls, *args, **kwargs):
+            # type: (Any, Any, Any, Any) -> None
             if cls == SentryAsgiMiddleware:
-                return old_middleware_init(self, cls, **options)
+                return old_middleware_init(self, cls, *args, **kwargs)
 
             span_enabled_cls = _enable_span_for_middleware(cls)
-            old_middleware_init(self, span_enabled_cls, **options)
+            old_middleware_init(self, span_enabled_cls, *args, **kwargs)
 
             if cls == AuthenticationMiddleware:
                 patch_authentication_middleware(cls)
@@ -362,6 +398,11 @@ def patch_asgi_app():
             mechanism_type=StarletteIntegration.identifier,
             transaction_style=integration.transaction_style,
             span_origin=StarletteIntegration.origin,
+            http_methods_to_capture=(
+                integration.http_methods_to_capture
+                if integration
+                else DEFAULT_HTTP_METHODS_TO_CAPTURE
+            ),
         )
 
         middleware.__call__ = middleware._run_asgi3
@@ -404,10 +445,12 @@ def patch_request_response():
                 request = args[0]
 
                 _set_transaction_name_and_source(
-                    Scope.get_current_scope(), integration.transaction_style, request
+                    sentry_sdk.get_current_scope(),
+                    integration.transaction_style,
+                    request,
                 )
 
-                sentry_scope = Scope.get_isolation_scope()
+                sentry_scope = sentry_sdk.get_isolation_scope()
                 extractor = StarletteRequestExtractor(request)
                 info = await extractor.extract_request_info()
 
@@ -440,14 +483,20 @@ def patch_request_response():
 
         else:
 
-            @ensure_integration_enabled(StarletteIntegration, old_func)
+            @functools.wraps(old_func)
             def _sentry_sync_func(*args, **kwargs):
                 # type: (*Any, **Any) -> Any
                 integration = sentry_sdk.get_client().get_integration(
                     StarletteIntegration
                 )
-                sentry_scope = Scope.get_isolation_scope()
+                if integration is None:
+                    return old_func(*args, **kwargs)
 
+                current_scope = sentry_sdk.get_current_scope()
+                if current_scope.transaction is not None:
+                    current_scope.transaction.update_active_thread()
+
+                sentry_scope = sentry_sdk.get_isolation_scope()
                 if sentry_scope.profile is not None:
                     sentry_scope.profile.update_active_thread_id()
 
@@ -515,7 +564,9 @@ def patch_templates():
             # type: (Jinja2Templates, *Any, **Any) -> None
             def add_sentry_trace_meta(request):
                 # type: (Request) -> Dict[str, Any]
-                trace_meta = Markup(Scope.get_current_scope().trace_propagation_meta())
+                trace_meta = Markup(
+                    sentry_sdk.get_current_scope().trace_propagation_meta()
+                )
                 return {
                     "sentry_trace_meta": trace_meta,
                 }
@@ -630,8 +681,10 @@ class StarletteRequestExtractor:
         # type: (StarletteRequestExtractor) -> Optional[Dict[str, Any]]
         if not self.is_json():
             return None
-
-        return await self.request.json()
+        try:
+            return await self.request.json()
+        except JSONDecodeError:
+            return None
 
 
 def _transaction_name_from_router(scope):
@@ -643,13 +696,17 @@ def _transaction_name_from_router(scope):
     for route in router.routes:
         match = route.matches(scope)
         if match[0] == Match.FULL:
-            return route.path
+            try:
+                return route.path
+            except AttributeError:
+                # routes added via app.host() won't have a path attribute
+                return scope.get("path")
 
     return None
 
 
 def _set_transaction_name_and_source(scope, transaction_style, request):
-    # type: (Scope, str, Any) -> None
+    # type: (sentry_sdk.Scope, str, Any) -> None
     name = None
     source = SOURCE_FOR_STYLE[transaction_style]
 
@@ -663,7 +720,7 @@ def _set_transaction_name_and_source(scope, transaction_style, request):
 
     if name is None:
         name = _DEFAULT_TRANSACTION_NAME
-        source = TRANSACTION_SOURCE_ROUTE
+        source = TransactionSource.ROUTE
 
     scope.set_transaction_name(name, source=source)
     logger.debug(
@@ -678,9 +735,9 @@ def _get_transaction_from_middleware(app, asgi_scope, integration):
 
     if integration.transaction_style == "endpoint":
         name = transaction_from_function(app.__class__)
-        source = TRANSACTION_SOURCE_COMPONENT
+        source = TransactionSource.COMPONENT
     elif integration.transaction_style == "url":
         name = _transaction_name_from_router(asgi_scope)
-        source = TRANSACTION_SOURCE_ROUTE
+        source = TransactionSource.ROUTE
 
     return name, source
